/* Copyright (c) 2009 Peter Troshin
 * Copyright (c) 2013 Alexander Sherstnev
 *  
 *  JAva Bioinformatics Analysis Web Services (JABAWS) @version: 1.0
 * 
 *  This library is free software; you can redistribute it and/or modify it under the terms of the
 *  Apache License version 2 as published by the Apache Software Foundation
 * 
 *  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Apache 
 *  License for more details.
 * 
 *  A copy of the license is in apache_license.txt. It is also available here:
 * @see: http://www.apache.org/licenses/LICENSE-2.0.txt
 * 
 * Any republication or derived work distributed in source code form
 * must include this copyright and license notice.
 */

package compbio.data.sequence;

import java.util.List;
import java.util.ArrayList;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;

import compbio.util.annotation.Immutable;
import compbio.data.sequence.Alignment;
/**
 * Multiple sequence alignment and Jpred prediction.
 * 
 * Does not give any guarantees on the content of individual FastaSequece
 * records. It does not guarantee neither the uniqueness of the names of
 * sequences nor it guarantees the uniqueness of the sequences.
 * 
 * @see FastaSequence
 * @see AlignmentMetadata
 * 
 * @author pvtroshin
 * 
 * @version 1.0 September 2009
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
@Immutable
public final class JpredAlignment extends Alignment{

	private AlignmentMetadata metadata;
	private List<FastaSequence> sequences;
	
	FastaSequence jnetpred;
	FastaSequence JNETCONF;
	FastaSequence JNETSOL25;
	FastaSequence JNETSOL5;
	FastaSequence JNETSOL0;
	FastaSequence JNETHMM;
	FastaSequence JNETPSSM;
	FastaSequence JNETJURY;

	private JpredAlignment() {
		// This has to has a default constructor for JaxB
	}

	/**
	 * @param sequences
	 * @param program
	 * @param gapchar
	 */
	public JpredAlignment(List<FastaSequence> sequences, Program program,
			char gapchar) {
		DecomposeInitalJpredSeq(sequences);
		this.metadata = new AlignmentMetadata(Program.CLUSTAL, gapchar);
	}

	/**
	 * 
	 * @param sequences
	 * @param metadata
	 */
	public JpredAlignment(List<FastaSequence> sequences, AlignmentMetadata metadata) {
		DecomposeInitalJpredSeq(sequences);
		this.metadata = metadata;
	}

	/**
	 * 
	 * @param sequences
	 */
	private void DecomposeInitalJpredSeq(List<FastaSequence> sequences) {
		List<FastaSequence> realsequences = new ArrayList<FastaSequence>();
		for (FastaSequence s : sequences) {
			if (s.getId().equals("jnetpred")) {
				this.jnetpred = s;
			} else if (s.getId().equals("JNETCONF")) {
				this.JNETCONF = s;
			} else if (s.getId().equals("JNETSOL25")) {
				this.JNETSOL25 = s;
			} else if (s.getId().equals("JNETSOL5")) {
				this.JNETSOL5 = s;
			} else if (s.getId().equals("JNETSOL0")) {
				this.JNETSOL0 = s;
			} else if (s.getId().equals("JNETHMM")) {
				this.JNETSOL0 = s;
			} else if (s.getId().equals("JNETHMM")) {
				this.JNETHMM = s;
			} else if (s.getId().equals("JNETSOL0")) {
				this.JNETPSSM = s;
			} else if (s.getId().equals("JNETHMM")) {
				this.JNETJURY = s;
			} else {
				realsequences.add(s);
			}
		}
		this.sequences = realsequences;
	}

	/**
	 * 
	 * @return Jpred prediction 
	 */
	public String getJpredPrediction() {
		return this.jnetpred.getSequence();
	}

	/**
	 * 
	 * @return Jpred JNETSOL25 line
	 */
	public String getJpredSOL25() {
		return this.JNETSOL25.getSequence();
	}

	/**
	 * 
	 * @return Jpred JNETSOL5 line
	 */
	public String getJpredSOL5() {
		return this.JNETSOL5.getSequence();
	}
	/**
	 * 
	 * @return Jpred JNETSOL0 line
	 */
	public String getJpredSOL0() {
		return this.JNETSOL0.getSequence();
	}
	/**
	 * 
	 * @return Jpred JNETCONF line
	 */
	public String getJpredJNETCONF() {
		return this.JNETCONF.getSequence();
	}
	/**
	 * 
	 * @return list of FastaSequence records
	 */
	public List<FastaSequence> getSequences() {
		return sequences;
	}

	/**
	 * 
	 * @return a number of sequence in the alignment
	 */
	public int getSize() {
		return this.sequences.size();
	}

	/**
	 * 
	 * @return AlignmentMetadata object
	 */
	public AlignmentMetadata getMetadata() {
		return metadata;
	}

	@Override
	public String toString() {
		String sseq = "";
		for (FastaSequence fs : getSequences()) {
			sseq += fs.toString() + "\n";
		}
		return sseq;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((metadata == null) ? 0 : metadata.hashCode());
		result = prime * result
				+ ((sequences == null) ? 0 : sequences.hashCode());
		return result;
	}

	/**
	 * Please note that this implementation does not take the order of sequences
	 * into account!
	 */
	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Alignment)) {
			return false;
		}
		Alignment al = (Alignment) obj;
		if (this.getSize() != al.getSize()) {
			return false;
		}
		if (!this.getMetadata().equals(al.getMetadata())) {
			return false;
		}
		int outerCounter = 0;
		int matchCounter = 0;
		for (FastaSequence fs : getSequences()) {
			outerCounter++;
			for (FastaSequence fs1 : al.getSequences()) {
				if (fs.equals(fs1)) {
					matchCounter++;
					continue;
				}
			}
			// Match for at lease one element was not found!
			if (outerCounter != matchCounter) {
				return false;
			}
		}

		return true;
	}

}
